@@ -1,11 +1,11 @@
 context("Credential providers")
 
-test_that("r_env_provider", {
+test_that("env_provider", {
   Sys.setenv(
     "AWS_ACCESS_KEY_ID" = "foo",
     "AWS_SECRET_ACCESS_KEY" = "bar"
   )
-  creds <- r_env_provider()
+  creds <- env_provider()
   expect_equal(creds$access_key_id, "foo")
   expect_equal(creds$secret_access_key, "bar")
 
@@ -14,7 +14,7 @@ test_that("r_env_provider", {
     "AWS_SECRET_ACCESS_KEY" = "bar",
     "AWS_SESSION_TOKEN" = "foobar"
   )
-  creds <- r_env_provider()
+  creds <- env_provider()
   expect_equal(creds$access_key_id, "foo")
   expect_equal(creds$secret_access_key, "bar")
   expect_equal(creds$session_token, "foobar")
@@ -24,6 +24,6 @@ test_that("r_env_provider", {
     "AWS_SECRET_ACCESS_KEY" = "",
     "AWS_SESSION_TOKEN" = ""
   )
-  creds <- r_env_provider()
+  creds <- env_provider()
   expect_null(creds)
 })
